--[[

   game.lua
   Handles the main platform game of Because it's fun, Fay.
	
   Copyright 2008 Oxeye Game Studio. http://www.oxeyegames.com
	
   File contributors:
      Jens Bergensten
      Daniel Brynolf

   This game was written for the TIGSource Procedural Generation
   competition in May 2008. You may use this Lua source code in
   any way you please, we don't mind. However, the game's sound
   effects and artwork belong to us, so please ask for permission
   before you do anything funny =)

]]

dofile("world.lua")
dofile("monsters.lua")

local randomSeed = 0
local gameField = {}
local gameFieldSprites = {}
local gameFieldRotated = {}
local DIMA = 32  -- x width
local DIMB = 256 -- y height
local playerFay = {}
local playerCorn = {}
local keys = {}
local jumpers = {}
local doors = {}
local doodads = {}
local cornShots = {}
local fxItems = {}
local powerups = {} -- filled by monsters.lua
local powerupTimer = 0 -- animation

local introMusicTimer = 4
local extro = { started = false, timer = 15 }
local gameOver = { timer = 0, musicStarted = false }
local activePlayer = 0
local screenInfo = { prevToggle = false, switchPos = 0, isSwitching = false }

--local blockSprite = nil
local jumperSprites = {}
local circleSprite = nil
local bgSprite = nil
local cornSprite = {}
local faySprite = {}
local fxSprite = {}
local powerupSprites = powerupSprites
local doorSprites = {}

function addFx(sprite, x, y, vx, vy, sSize, eSize, sAlpha, eAlpha, life, spinSpeed, updateFunction, gravity, startSpin)
	table.insert(fxItems, {
		sprite=sprite,
		x=x, y=y,
		vx=vx, vy=vy,
		size=sSize, sSize=sSize, eSize=eSize,
		alpha=sAlpha, sAlpha=sAlpha, eAlpha=eAlpha,
		life=life, totalLife=life,
		rotated = false,
		spin = startSpin or 0,
		spinSpeed = spinSpeed or 0,
		updateFunction = updateFunction,
		cooldown = 0,
		gravity = gravity or 0
		
		})
end

local function addFxRotated(sprite, x, y, vx, vy, sSize, eSize, sAlpha, eAlpha, life, spinSpeed, updateFunction, gravity, startSpin)
	table.insert(fxItems, {
		sprite=sprite,
		x=x, y=y,
		vx=vx, vy=vy,
		size=sSize, sSize=sSize, eSize=eSize,
		alpha=sAlpha, sAlpha=sAlpha, eAlpha=eAlpha,
		life=life, totalLife=life,
		rotated = true,
		spin = startSpin or 0,
		spinSpeed = spinSpeed or 0,
		updateFunction = updateFunction,
		cooldown = 0,
		gravity = gravity or 0
		})
end

function fxBloodGusher(fx, time)
	if fx.cooldown <= 0 then
		addFx(fxSprite.blood, fx.x, fx.y, math.cos(fx.spin+math.random()*1-0.5)*math.random()*200 ,
		-math.sin(fx.spin+math.random()*1-0.5)*math.random()*200, 0.2, 1.3, 255, 0, math.random()*0.9, 1.5)
		fx.cooldown = 0.01 + math.random()*0.05
	else 
	fx.vx = fx.vx - fx.vx*time*2.5
	fx.vy = fx.vy - fx.vy*time*2.5
	
	fx.cooldown = fx.cooldown - time
	end

end

local function updateFx(time)
	for index,fx in pairs(fxItems) do
		fx.life = fx.life - time
		if fx.life > 0 then
			if fx.gravity == 0 then
				
				fx.x = fx.x + fx.vx * time
				fx.y = fx.y + fx.vy * time
			else
				fx.vy = fx.vy + fx.gravity*time
				if gameField.isPositionBlocked(math.ceil((fx.x+ fx.vx * time)/32),math.ceil((fx.y)/32)) then
					fx.vx = fx.vx * -0.5
					fx.x = fx.x + fx.vx * time
				else
					fx.x = fx.x + fx.vx * time
				end
				
				if gameField.isPositionBlocked(math.ceil((fx.x)/32),math.ceil((fx.y + fx.vy * time)/32)) then
					fx.vy = fx.vy *-0.5
					fx.y = fx.y + fx.vy * time
				else
					fx.y = fx.y + fx.vy * time
				end
				
				fx.y = fx.y + fx.vy * time
			
			end
			fx.spin = fx.spin + fx.spinSpeed*time
			if fx.updateFunction ~= nil then
				fx.updateFunction(fx, time)
			end
			
			local scale = fx.life / fx.totalLife
			fx.size = fx.sSize * scale + fx.eSize * (1 - scale)
			fx.alpha = fx.sAlpha * scale + fx.eAlpha * (1 - scale)
		else
			table.remove(fxItems, index)
		end
	end
end

-- this function is global so that monsters.lua may call it
function notifyCornShotHit(x, y, monsterWasKilled)
	addFxRotated(cornSprite.magicHit, x, y, 0, 0, 0, 1, 255, 100, 0.2)

	if monsterWasKilled then
		playerCorn.powerMeter = 5
		playerCorn.powerRefreshTimer = 1
		daisy.playSound("cornHehe" .. math.random(1,4) .. ".wav")
	end
end

local function openDoor(cx, cy, kind)
	if gameField[cx][cy - 1] == kind then
		gameField[cx][cy - 1] = 0
		gameFieldRotated[cy - 1][cx] = 0
		--table.remove(gameFieldSprites[cx][cy - 1], #(gameFieldSprites[cx][cy - 1]))
	elseif gameField[cx][cy + 1] == kind then
		gameField[cx][cy + 1] = 0
		gameFieldRotated[cy + 1][cx] = 0
		--table.remove(gameFieldSprites[cx][cy + 1], #(gameFieldSprites[cx][cy + 1]))
	end
	gameField[cx][cy] = 0
	gameFieldRotated[cy][cx] = 0
	--table.remove(gameFieldSprites[cx][cy], #(gameFieldSprites[cx][cy]))
	daisy.playSound("door.wav")
	
	-- locate the door
	for index, door in pairs(doors) do
		local dx = cx * 32 - 16
		local dy = cy * 32
		
		if not door.open then
			if door.x >= (dx - 1) and door.x <= (dx + 1) and door.y >= (dy - 32) and door.y <= (dy + 32) then
				door.open = true
				break
			end
		end
	end
end

-- FFFFFFFFFF    AA    YY      YY
-- FF          AA  AA  YY      YY
-- FFFFFF    AA      AA  YY  YY
-- FF        AAAAAAAAAA    YY
-- FF        AA      AA    YY
-- FF        AA      AA    YY
local function updatePlayerFay(time)
	local player = playerFay
	local field = gameField

	if player.hurtTimer > 0 then
		player.hurtTimer = player.hurtTimer - time
	end
	local mayMove = player.hurtTimer < 1
	player.screenShakeTimer = player.screenShakeTimer - time
	player.starTimer = player.starTimer - time
	if activePlayer == 0 or not singlePlayer then
		player.help.timer = player.help.timer - time
	end
	if player.health <= 2 then
		player.healthAnimation = player.healthAnimation + time
	else
		player.healthAnimation = .75
	end
	
	if player.isDead then
		player.left, player.right, player.jump, player.crouch = false, false, false, false
		mayMove = false
		player.isAttached = false
		
		if player.hurtTimer <= 0 and player.numDaisies > 0 then
			player.isDead = false
			player.health = 6
			player.hurtTimer = 1 -- respawn invincible
			player.numDaisies = player.numDaisies - 1
			for index,powerup in pairs(playerPowerups.fay) do
				if powerup == 6 then
					table.remove(playerPowerups.fay, index)
					break
				end
			end
			daisy.playSound("aah.ogg")
		end
	end

	local previousX,previousY = player.x, player.y

	if player.jumpTimer > 0 then
		player.jumpTimer = player.jumpTimer - time
	end
	if (not player.jump or player.jumpTimer <= 0) and not player.isAttached then
		player.vy = player.vy + time * 1600
	end
	
	player.leaning = player.leaning - 10.5*player.leaning*time
	
	if player.touchesGround then
		player.leaning = player.leaning + player.vx*time*0.010
	elseif player.vx > 0  then
		player.leaning = player.leaning - player.vy*time*0.005
	elseif player.vx < 0  then
		player.leaning = player.leaning + player.vy*time*0.005
	end

	if player.left and mayMove then
		if not player.isAttached then
			player.vx = player.vx + (-300 - player.vx) * time * 7
		end
		player.facingLeft = true
	end
	if player.right and mayMove then
		if not player.isAttached then
			player.vx = player.vx + (300 - player.vx) * time * 7
		end
		player.facingLeft = false
	end
	if not mayMove then
		player.vx = player.vx - player.vx * time * 3
	elseif not player.left and not player.right then
		player.vx = player.vx - player.vx * time * 10
	end
	if player.jump and player.touchesGround and not player.preventBunny and not player.isAttached and mayMove then
		player.vy = -400
		player.jumpTimer = 0.3
		player.touchesGround = false
		player.preventBunny = true
		daisy.playSound("jump" .. math.random(1,2) .. ".wav")
		--daisy.playSound("jump1.wav")
	end
	if not player.jump then
		player.preventBunny = false
	end

	if player.isAttached and mayMove then
		if player.left then
			player.x = player.x - time * 200
		elseif player.right then
			player.x = player.x + time * 200
		end
		if player.jump then
			player.y = player.y - time * 200
		elseif player.crouch then
			player.y = player.y + time * 200
		end
	else
		player.x = player.x + time * player.vx
		player.y = player.y + time * player.vy
	end

	local floorRight = field.isPositionBlocked(math.ceil((player.x + 9) / 32), math.ceil(player.y / 32))
	local floorLeft = field.isPositionBlocked(math.ceil((player.x - 9) / 32), math.ceil(player.y / 32))

	if (floorRight or floorLeft) and (not player.isAttached or player.crouch or player.vy > 0) then
		-- check if the previous X position was better
		floorRight = field.isPositionBlocked(math.ceil((previousX + 9) / 32), math.ceil(player.y / 32))
		floorLeft = field.isPositionBlocked(math.ceil((previousX - 9) / 32), math.ceil(player.y / 32))

		if not floorRight and not floorLeft then
			player.x = previousX
		else
			if player.vy > 750 then
				player.vx = 0
				addFx(circleSprite, player.x, player.y, math.random() * -80, math.random() * -10, 0.1, 0.5, 200, 50, 0.5)
				addFx(circleSprite, player.x, player.y, math.random() * 80, math.random() * -10, 0.1, 0.5, 200, 50, 0.5)
				daisy.playSound("land2.wav")
			elseif player.vy > 100 then
				local volume = player.vy / 750
				daisy.playSound("land.wav", volume)
			end
			player.touchesGround = true
			player.vy = 0
			player.y = player.y - player.y % 32
		end
	elseif player.vy > 0 then
		player.touchesGround = false
	end

	local ceilRight = field.isPositionBlocked(math.ceil((player.x + 9) / 32), math.ceil((player.y - 32) / 32))
	local ceilLeft = field.isPositionBlocked(math.ceil((player.x - 9) / 32), math.ceil((player.y - 32) / 32))
	if (ceilRight or ceilLeft) and (not player.isAttached or player.jump or player.vy < 0) then
		-- check if the previous X position was better
		ceilRight = field.isPositionBlocked(math.ceil((previousX + 9) / 32), math.ceil((player.y - 32) / 32))
		ceilLeft = field.isPositionBlocked(math.ceil((previousX - 9) / 32), math.ceil((player.y - 32) / 32))

		if not ceilRight and not ceilLeft then
			player.x = previousX
		else
			if player.vy < -200 then
				local volume = -player.vy / 900
				if volume > 1 then
					volume = 1
				end
				daisy.playSound("ceilinghit.wav", volume)
			end
			player.vy = 0
			player.y = math.ceil(player.y / 32) * 32
			player.jumpTime = 0
		end
	end

	local checkY = math.ceil((player.y - 1) / 32)
	local checkXRight = math.ceil((player.x + 10) / 32)
	local checkXLeft = math.ceil((player.x - 10) / 32)
	if (player.vx > 0 or (player.isAttached and player.right)) and
		(field.isPositionBlocked(checkXRight, math.ceil((player.y - 31) / 32)) or
		field.isPositionBlocked(checkXRight, checkY)) then

		player.vx = 0
		player.x = player.x - player.x % 32 + 22

		if player.carriedKey and field[checkXRight][checkY] == player.carriedKey.type then

			openDoor(checkXRight, checkY, player.carriedKey.type)

			for index,k in pairs(keys) do
				if k == player.carriedKey then
					table.remove(keys, index)
					break
				end
			end
			player.carriedKey = nil
		end

	elseif (player.vx < 0 or (player.isAttached and player.left)) and
		(field.isPositionBlocked(checkXLeft, math.ceil((player.y - 31) / 32)) or
		field.isPositionBlocked(checkXLeft, math.ceil((player.y - 1) / 32))) then

		player.vx = 0
		player.x = math.floor(player.x / 32) * 32 + 10

		if player.carriedKey and field[checkXLeft][checkY] == player.carriedKey.type then
			openDoor(checkXLeft, checkY, player.carriedKey.type)

			for index,k in pairs(keys) do
				if k == player.carriedKey then
					table.remove(keys, index)
					break
				end
			end
			player.carriedKey = nil
		end
	end

	if player.vy > 0 then
		for index,jumper in pairs(jumpers) do
			if player.y > (jumper.posY - 10) and player.y < jumper.posY and player.x > (jumper.posX - 20) and player.x < (jumper.posX + 20) then
				player.vy = -player.vy
				if player.jump then
					player.vy = player.vy - 200
				else
					player.vy = player.vy + 200
					if player.vy > -400 then
						player.vy = -400
					end
				end
				if player.vy < -1500 then
					player.vy = -1500
				end
				--print(player.vy)
				local pitch = 1.0 + (-(player.vy + 400) / 2500.0)
				if pitch > 2.0 then
					pitch = 2.0
				end
				local volume = 0.2 + pitch * 0.4
				daisy.playSound("bounce.wav", volume, 0.0, pitch)

				player.jumpTimer = 0
				player.touchesGround = false
				player.preventBunny = true
			end
		end
	end

	if not player.isAttached and gameField[math.ceil(player.x / 32)][math.ceil(player.y / 32)] == -1 then
		player.isAttached = true
		player.vx = 0
		player.vy = 0
		player.touchesGround = false
	elseif player.isAttached and gameField[math.ceil(player.x / 32)][math.ceil(player.y / 32)] ~= -1 then
		if player.jump then
			player.vy = -400
			player.jumpTimer = 0.3
			player.touchesGround = false
			player.preventBunny = true
			daisy.playSound("jump" .. math.random(1,2) .. ".wav")
			--daisy.playSound("jump1.wav")
		end
		if player.left then
			player.vx = -200
		elseif player.right then
			player.vx = 200
		end
		player.isAttached = false
	end

	if player.touchesGround and not player.isAttached and (player.left or player.right) then
		player.stepTimer = player.stepTimer - time * math.abs(player.vx)
		if player.stepTimer <= 0 then
			player.stepTimer = 30
			local index = math.random(1,2)
			local pitch = 0.9 + math.random() * 0.25
			daisy.playSound("step" .. index .. ".wav", 1.0, 0.0, pitch)
		end
	elseif player.isAttached and (player.left or player.right or player.jump or player.crouch) then
		player.stepTimer = player.stepTimer - time * 200
		if player.stepTimer <= 0 then
			player.stepTimer = 30
			daisy.playSound("grass" .. math.random(1,8) .. ".wav")
		end
	end

	if player.carriedKey then
		player.carriedKey.posX = player.x + math.cos(player.leaning*-1 + math.pi*0.5)*55
		player.carriedKey.posY = player.y - math.sin(player.leaning*-1 + math.pi*0.5)*55
	end
	if player.keyTimer > 0 then
		player.keyTimer = player.keyTimer - time
	end
	if player.crouch and player.keyTimer <= 0 then
		-- look for keys to pick up
		local found = false
		for index,key in pairs(keys) do
			if key ~= player.carriedKey then
				if key.posX > player.x - 20 and key.posX < player.x + 20 and key.posY > player.y - 30 and key.posY < player.y + 30 then
					-- pickup key
					if player.carriedKey then
						player.carriedKey.posX = key.posX
						player.carriedKey.posY = key.posY
						player.carriedKey.active = false
					end
					player.carriedKey = key
					key.active = false
					daisy.playSound("keys.wav")
					playerFay.keyHelpDisplayed = true
					found = true
					break
				end
			end
		end

		if not found and player.carriedKey and not player.isAttached  then
			-- throw key
			local key = player.carriedKey
			player.carriedKey = nil

			key.vy = 0
			if player.facingLeft then
				key.vx = -400
			else
				key.vx = 400
			end
			key.vy = -100
			key.vx = key.vx + player.vx
			if player.vy < 0 then
				key.vy = key.vy - 150
			end
			key.active = true
			
			-- if the key's position's blocked, we throw it from Fay's center
			if field.isPositionBlocked(math.ceil(key.posX / 32), math.ceil(key.posY / 32)) then
				key.posY = key.posY + 32
			end

			daisy.playSound("throw.wav")
		end

		player.keyTimer = .5
	end

	if player.y < (32*19) and player.starTimer < 0 then
		player.starTimer = 0.05 + math.random() * 0.3
		addFx(powerupSprites.star, player.x + math.random(-20, 20), player.y + math.random(-40, 5), 0, math.random() * -300, 0.8, 0.7, 200, 255, 0.5)

		if playerCorn.x < (32*19) and not extro.started then
			extro.started = true
			daisy.playMusic("quakeLoop.wav", 0.8, true)
		end
	end
end
local function hurtFay(amount, sourceX, sourceY)

	local player = playerFay

	if player.hurtTimer <= 0 and not player.isDead then
		player.hurtTimer = 2
		if sourceX < player.x then
			player.vx = 200
		else
			player.vx = -200
		end
		player.vy = player.vy - 200
		daisy.playSound("ouch" .. math.random(1,3) .. ".wav")

		player.health = player.health - amount

		if player.health <= 0 then
			player.hurtTimer = 5
			player.isDead = true
			
			if player.numDaisies > 0 then
				daisy.playSound("chimes.wav")
			end
			
			if player.carriedKey then
				player.carriedKey.active = true
				player.carriedKey.vx = player.vx
				player.carriedKey.vy = player.vy
				player.carriedKey = nil
			end
		end
	end

end

--   CCCCCC      OOOOOO    RRRRRRRR    NN      NN
-- CC      CC  OO      OO  RR      RR  NNNN    NN
-- CC          OO      OO  RR      RR  NN  NN  NN
-- CC          OO      OO  RRRRRRRR    NN  NN  NN
-- CC      CC  OO      OO  RR      RR  NN    NNNN
--   CCCCCC      OOOOOO    RR      RR  NN      NN

local function updatePlayerCorn(time)
	local player = playerCorn
	local field = gameFieldRotated

	if player.hurtTimer > 0 then
		player.hurtTimer = player.hurtTimer - time
	end
	if player.health <= 2 then
		player.healthAnimation = player.healthAnimation + time * 2
	else
		player.healthAnimation = player.healthAnimation + time
	end
	player.screenShakeTimer = player.screenShakeTimer - time
	player.starTimer = player.starTimer - time
	player.powerRefreshTimer = player.powerRefreshTimer - time
	if activePlayer == 1 or not singlePlayer then
		player.help.timer = player.help.timer - time
	end

	local mayMove = player.hurtTimer < 1
	
	if player.isDead then
		player.left, player.right, player.jump, player.crouch = false, false, false, false
		mayMove = true -- hack to make corn fall downwards even though hurtTimer > 1
		
		if player.hurtTimer <= 0 and player.numDaisies > 0 then
			player.isDead = false
			player.health = 6
			player.hurtTimer = 1 -- respawn invincible
			player.numDaisies = player.numDaisies - 1
			for index,powerup in pairs(playerPowerups.corn) do
				if powerup == 6 then
					table.remove(playerPowerups.corn, index)
					break
				end
			end
			daisy.playSound("aah.ogg")
		end
	end

	player.jump = player.jump or player.crouch

	if not mayMove then
	elseif not player.jump then
		player.vy = player.vy + time * 700
		if (player.left or player.right) and player.vy > 0 then
			player.vy = player.vy + ((300 - math.abs(player.vx)) - player.vy) * time * 15
		else
		end
		player.flyHeld = false
	else

		player.vy = player.vy - player.vy * time * 2.5
		if player.vy < -400 then
			player.vy = -400
		end

		player.flapPushTimer = player.flapPushTimer - time
		if player.flapPushTimer <= 0 then
			player.vy = player.vy - 150
			player.flapPushTimer = 0.1 + math.random() * .15

		end
	end

	do
		local volume = 0.3
		local pitch = 0.8

		player.animFlap = player.animFlap + (math.abs(player.vx) + math.abs(player.vy))*time +85*time
		player.animLimbs = player.animLimbs + (math.abs(player.vx) + math.abs(player.vy))*time
		if player.jump then
			volume = 0.6
			pitch = 1.1
			player.animFlap = player.animFlap + (math.abs(player.vx) + math.abs(player.vy))*time*3
			player.animLimbs = player.animLimbs + (math.abs(player.vx) + math.abs(player.vy))*time*3
		end
		player.flapTimer = player.flapTimer - time
		if player.flapTimer <= 0 and player.groundTimer > 0 then
			player.flapTimer = 0.2
			daisy.playSound("wing-" .. math.random(1,6) .. ".wav", volume, 0, pitch)
			--player.animFlap = player.animFlap + math.random(2,7)+5
		end
	end

	player.leaning = player.leaning *0.99
	player.animDirection = 0
	if player.left and mayMove then
		player.leaning = player.leaning - 1*time
		player.vx = player.vx + (-380 - player.vx) * time * 2
		player.facingLeft = true
		player.animDirection = -1
	end
	if player.right and mayMove then
		player.leaning = player.leaning + 1*time
		player.vx = player.vx + (380 - player.vx) * time * 2
		player.facingLeft = false
		player.animDirection = 1
	end
	if not mayMove or (not player.right and not player.left) then
		if player.leaning < -.01 and player.facingLeft then
			player.animDirection = -1
		elseif player.leaning > .01 and not player.facingLeft then
			player.animDirection = 1
		end
	end
	if not mayMove then
		player.vx = player.vx - player.vx * time * 3
		player.vy = player.vy - player.vy * time * 3
	elseif (not player.left and not player.right) then
		player.vx = player.vx - player.vx * time * 10
	end

	if player.animLimbs > 2*math.pi*95 then
		player.animLimbs = 0
	end

	if player.vx > -.1 and player.vx < .1 and player.groundTimer <= 0 and mayMove then
		player.animLimbs = player.animLimbs*0.96 + 2*math.pi*24*0.04
	end

	player.x = player.x + time * player.vx
	player.y = player.y + time * player.vy

	local floorRight = field.isPositionBlocked(math.ceil((player.x + 9) / 32), math.ceil(player.y / 32))
	local floorLeft = field.isPositionBlocked(math.ceil((player.x - 9) / 32), math.ceil(player.y / 32))

	if floorRight or floorLeft then
		if player.vy > 750 then
			player.vx = 0
		end
		player.groundTimer = 0
		player.vy = 0
		player.y = player.y - player.y % 32
	else --if player.vy > 0 then
		--player.touchesGround = false
		player.groundTimer = player.groundTimer + time
	end

	local ceilRight = field.isPositionBlocked(math.ceil((player.x + 9) / 32), math.ceil((player.y - 32) / 32))
	local ceilLeft = field.isPositionBlocked(math.ceil((player.x - 9) / 32), math.ceil((player.y - 32) / 32))
	if ceilRight or ceilLeft then
		player.vy = 0
		player.y = math.ceil(player.y / 32) * 32
		player.jumpTime = 0
	end

	if player.vx > 0 and
		(field.isPositionBlocked(math.ceil((player.x + 10) / 32), math.ceil((player.y - 31) / 32)) or
		field.isPositionBlocked(math.ceil((player.x + 10) / 32), math.ceil((player.y - 1) / 32))) then

		player.vx = 0
		player.x = player.x - player.x % 32 + 22
	elseif player.vx < 0 and
		(field.isPositionBlocked(math.ceil((player.x - 10) / 32), math.ceil((player.y - 31) / 32)) or
		field.isPositionBlocked(math.ceil((player.x - 10) / 32), math.ceil((player.y - 1) / 32))) then

		player.vx = 0
		player.x = math.floor(player.x / 32) * 32 + 10
	end

	if player.crouch and player.powerMeter >= 1 and not player.attackStarted and mayMove then
		player.attackStarted = true

		player.attackTimer = player.attackTimer + .2
		player.powerMeter = player.powerMeter - 1

		local startVX = 800
		local left,top,right,bottom = player.x,player.y - 100,player.x,player.y + 100
		if player.facingLeft then
			startVX = -800
			left = left - 300
		else
			right = right + 300
		end

		local x = player.x
		local y = player.y - 16

		-- check for monsters in the area
		local sx = x
		local sy = y
		local targetMonster = nil
		local monsters = getMonsters()
		local life = 0.75
		for mi,monster in pairs(monsters) do
			if monster.y > left and monster.y < right and monster.x > top and monster.x < bottom then
				targetMonster = monster
				startVX = startVX * .25
				life = 1.5
				break
			end
		end
		local angle = 0
		if player.facingLeft then
			angle = math.pi
		end

		table.insert(cornShots, {x=x, y=y, sx=x, sy=y, vx=startVX, vy = 100, life=life, monster=targetMonster, angle=angle})
		addFxRotated(cornSprite.magicGlow, x, y, 0, 0, 2, 0.25, 200, 0, 0.25)
		daisy.playSound("EagleLaunch" .. math.random(1,2) .. ".wav")

	elseif not player.crouch then
		player.attackStarted = false
	end

	if player.powerMeter < 5 and not player.attackStarted then
		player.powerMeter = player.powerMeter + time
		if player.powerMeter > 5 then
			player.powerMeter = 5
		end
	end

	if player.x < (32*19) and player.starTimer < 0 then
		player.starTimer = 0.05 + math.random() * 0.3
		addFxRotated(powerupSprites.star, player.x + math.random(-20, 20), player.y + math.random(-40, 5), 0, math.random() * -300, 0.8, 0.7, 200, 255, 0.5)
	end

end
local function hurtCorn(amount, sourceX, sourceY)
	local player = playerCorn

	if player.hurtTimer <= 0 and not player.isDead then
		player.hurtTimer = 2
		if sourceY < player.x then
			player.vx = 200
		else
			player.vx = -200
		end
		player.vy = player.vy - 200
		player.health = player.health - amount
		daisy.playSound("cornOuch" .. math.random(1,4) .. ".wav")
		
		if player.health <= 0 then
		
			if player.numDaisies > 0 then
				daisy.playSound("chimes.wav")
			end
			
			player.hurtTimer = 5
			player.isDead = true
		end
	end

end

local function makePlayers()

	playerFay = {
			x = DIMA * 32 - 124, y = DIMB * 32 - 124,
			vx = 0, vy = 0,
			touchesGround = false,
			jumpTimer = 0,
			preventBunny = false,
			left = false, right = false, jump = false, crouch = false,
			viewX = startX, viewY = startY,
			carriedKey = nil,
			keyTimer = 0,
			stepTimer = 0,
			facingLeft = true,
			isAttached = false,
			hurtTimer = 0,
			hurtMethod = hurtFay,
			health = 6,
			healthAnimation = 0,
			isDead = false, numDaisies = 0,
			screenShakeTimer = 0,
			starTimer = 0,
			animDirection = "front",
			leaning = 0,
			limbNames = {"head","body", "lArm", "rArm", "lLeg", "rLeg"},
			limbs = {},
			help = { timer = 0, text = "", width = 250 },
			keyHelpDisplayed = false
		}
	
	
	for index,limbName in pairs(playerFay.limbNames) do
		playerFay.limbs[limbName] = {d = 0, a = 0, r = 0, order = 10}	
	end
	
	playerCorn = {
			x = DIMB * 32 - 64, y = DIMA * 32 - 124,
			vx = 0, vy = 0,
			groundTimer = 0.2,
			left = false, right = false, jump = false, crouch = false,
			viewX = startX, viewY = startY,
			flapTimer = 0,
			flyHeld = false,
			flapPushTimer = 0,
			facingLeft = true,
			attackStarted = false,
			attackTimer = 0,
			powerMeter = 0, powerRefreshTimer = 0,
			hurtTimer = 0,
			hurtMethod = hurtCorn,
			health = 6,
			isDead = false, numDaisies = 0,
			healthAnimation = 0,
			animFlap = 0,
			leaning = 0,
			animLimbs = 0,
			animDirection = 0,
			screenShakeTimer = 0,
			starTimer = 0,
			help = { timer = 0, text = "", width = 250 }
		}
end

local function updateKeys(time)

	for index,key in pairs(keys) do
		if key.active then

			local monsters = getMonsters()
			
			for index,monster in pairs(monsters) do
				if key.posX > monster.x - 16 and key.posX < monster.x + 16 and key.posY > monster.y - 16 and key.posY < monster.y + 16 and monster.hurtTimer <= 0 then
				key.vx = key.vx * -0.5
				monster.stunTimer = 5.4
				end
			end
			
			local nx = key.posX + key.vx * time
			local ny = key.posY + key.vy * time
			local cx = math.ceil(nx / 32)
			local cy = math.ceil(ny / 32)
			local remove = false
			
			key.angle = key.angle + key.vx * time * .2

			if gameField.isPositionBlocked(cx, cy) then

				-- simple bounce check
				if cx == math.ceil(key.posX / 32) then
					key.vx = key.vx * .5
					key.vy = -key.vy * .5
				else
					key.vx = -key.vx * .5
					key.vy = key.vy * .5

					-- check if the collision position was a door matching the key
					if gameField[cx][cy] == key.type then
						openDoor(cx, cy, key.type)
						remove = true

					end
				end

				if math.abs(key.vx) < .7 and math.abs(key.vy) < 10 then
					key.active = false
				end

			else
				key.posX = nx
				key.posY = ny

				-- check for collision with fay
				if playerFay.keyTimer <= 0 and playerFay.carriedKey == nil then
					local dx,dy = nx - playerFay.x, ny - (playerFay.y - 16)

					if math.abs(dx) <= 10 and math.abs(dy) <= 20 then
						-- catch the key
						key.active = false
						playerFay.carriedKey = key
						daisy.playSound("keys.wav")
					end
				end
				key.vy = key.vy + time * 1000
			end

			if remove then
				table.remove(keys, index)
			end

		else
			if playerFay.keyTimer <= 0 and playerFay.carriedKey == nil then
				local dx,dy = key.posX - playerFay.x, key.posY - (playerFay.y - 16)

				if math.abs(dx) <= 10 and math.abs(dy) <= 25 then
					-- catch the key
					key.active = false
					playerFay.carriedKey = key
					daisy.playSound("keys.wav")
					
					if not playerFay.keyHelpDisplayed then
						playerFay.keyHelpDisplayed = true
						playerFay.help.timer = 7
						playerFay.help.width = 270
						playerFay.help.text = "Use Down Key to Throw Item"
						if not singlePlayer then
							playerFay.help.text = "Use S Key to Throw Item"
							playerFay.help.width = 220
						end
					end
				end
			end
			--key.angle = key.angle + time * math.pi * .3
		end
	end
end

local function updateDoors(time)
	for index, door in pairs(doors) do
		if door.open then
			door.anim = door.anim + time
			if door.anim > 1 then
				door.anim = 1
			end
		end
	end
end

local function recountPlayerDaisies()
	playerFay.numDaisies = 0
	for index, powerup in pairs(playerPowerups.fay) do
		if powerup == 6 then
			playerFay.numDaisies = playerFay.numDaisies + 1
		end
	end
	playerCorn.numDaisies = 0
	for index, powerup in pairs(playerPowerups.corn) do
		if powerup == 6 then
			playerCorn.numDaisies = playerCorn.numDaisies + 1
		end
	end
end

local function updatePowerups(time)
	powerupTimer = powerupTimer + time

	for index,powerup in pairs(powerups) do

		powerup.y = powerup.y - time
		local dx,dy = powerup.x - playerFay.x, powerup.y - (playerFay.y - 16)

		if math.abs(dx) <= 20 and math.abs(dy) <= 30 then
			for i=1,10 do
				local angle = math.random() * 2 * math.pi
				local vel = math.random(100, 500)
				addFx(powerupSprites.star, powerup.x, powerup.y, math.cos(angle) * vel, math.sin(angle) * vel, .5, 1.1, 255, 255, .25)
			end
			table.insert(playerPowerups.fay, powerup.type)
			table.remove(powerups, index)
			daisy.playSound("powerup.wav")
			
			if powerup.type == 6 then
				playerFay.numDaisies = playerFay.numDaisies + 1
			end
			break
		else
			dx,dy = powerup.x - (playerCorn.y - 16), powerup.y - playerCorn.x

			if math.abs(dx) <= 20 and math.abs(dy) <= 30 then
				for i=1,10 do
					local angle = math.random() * 2 * math.pi
					local vel = math.random(100, 500)
					addFxRotated(powerupSprites.star, powerup.y, powerup.x, math.cos(angle) * vel, math.sin(angle) * vel, .5, 1.1, 255, 255, .25)
				end
				table.insert(playerPowerups.corn, powerup.type)
				table.remove(powerups, index)
				daisy.playSound("powerup.wav")
				
				if powerup.type == 6 then
					playerCorn.numDaisies = playerCorn.numDaisies + 1
				end
				break
			end
		end

	end
end

local function updateCornShots(time)

	for index,shot in pairs(cornShots) do
		shot.life = shot.life - time
		if shot.life <= 0 then
			addFxRotated(cornSprite.magicHit, shot.x, shot.y, 0, 0, 1, 3, 128, 0, 0.5)
			daisy.playSound("TempestLaunch.wav")
			table.remove(cornShots, index)
		else
			shot.x = shot.x + shot.vx * time
			shot.y = shot.y + shot.vy * time

			shot.angle = shot.angle + math.sqrt(shot.vx * shot.vx + shot.vy * shot.vy) * time * .1

			if shot.monster then
				shot.sy = shot.monster.x
				shot.sx = shot.monster.y

				if (shot.x < shot.sx and shot.vx < 0) or (shot.x > shot.sx and shot.vx > 0) then
					shot.vx = shot.vx + (shot.sx - shot.x) * time * 32
				else
					shot.vx = shot.vx + (shot.sx - shot.x) * time * 16
				end
				if (shot.y < shot.sy and shot.vy < 0) or (shot.y > shot.sy and shot.vy > 0) then
					shot.vy = shot.vy + (shot.sy - shot.y) * time * 90
				else
					shot.vy = shot.vy + (shot.sy - shot.y) * time * 45
				end
			else
				shot.vx = shot.vx + (shot.sx - shot.x) * time * 16
				shot.vy = shot.vy + (shot.sy - shot.y) * time * 45
			end
		end
	end

end

local function updateView(time)
	if not singlePlayer then
		return
	end
	local toggle = daisy.isKeyPressed(32)
	if not screenInfo.prevToggle and toggle then
		if activePlayer == 0 then
			activePlayer = 1
			if not playerFay.isDead then
				daisy.playSound("fayCorn" .. math.random(1,2) .. ".wav")
			end
		else
			activePlayer = 0
			if not playerCorn.isDead then
				daisy.playSound("cornFay" .. math.random(1,4) .. ".wav")
			end
		end
		screenInfo.isSwitching = true
	end
	screenInfo.prevToggle = toggle

	if screenInfo.isSwitching then
		if activePlayer == 0 then
			screenInfo.switchPos = screenInfo.switchPos - time * 2
			if screenInfo.switchPos < 0 then
				screenInfo.switchPos = 0
				screenInfo.isSwitching = false
			end
		else
			screenInfo.switchPos = screenInfo.switchPos + time * 2
			if screenInfo.switchPos > 1 then
				screenInfo.switchPos = 1
				screenInfo.isSwitching = false
			end
		end
	end

end

-- UU      UU
-- UU      UU
-- UU      UU
-- UU      UU
-- UU      UU
--   UUUUUU   P D A T E
local function gameRun(deltaTime)

	if singlePlayer then
		if activePlayer == 0 then
			playerFay.left,playerFay.right,playerFay.jump,playerFay.crouch = daisy.isKeyPressed(37), daisy.isKeyPressed(39), daisy.isKeyPressed(38), daisy.isKeyPressed(40)
			playerCorn.left,playerCorn.right,playerCorn.jump,playerCorn.crouch = false, false, false, false
		else
			playerCorn.left,playerCorn.right,playerCorn.jump,playerCorn.crouch = daisy.isKeyPressed(37), daisy.isKeyPressed(39), daisy.isKeyPressed(38), daisy.isKeyPressed(40)
			playerFay.left,playerFay.right,playerFay.jump,playerFay.crouch = false, false, false, false
		end
	else
		playerCorn.left,playerCorn.right,playerCorn.jump,playerCorn.crouch = daisy.isKeyPressed(37), daisy.isKeyPressed(39), daisy.isKeyPressed(38), daisy.isKeyPressed(40)
		playerFay.left,playerFay.right,playerFay.jump,playerFay.crouch = daisy.isKeyPressed(65), daisy.isKeyPressed(68), daisy.isKeyPressed(87), daisy.isKeyPressed(83)
	end

	if deltaTime > .06 then
		deltaTime = .06
	end

	updatePlayerFay(deltaTime)
	updatePlayerCorn(deltaTime)
	updateKeys(deltaTime)
	updatePowerups(deltaTime)
	updateDoors(deltaTime)
	updateCornShots(deltaTime)
	updateMonsters(deltaTime)
	updateFx(deltaTime)

	if introMusicTimer > 0 then
		introMusicTimer = introMusicTimer - deltaTime
		if introMusicTimer <= 0 then
			daisy.stopMusic("29.mp3")
		else
			daisy.updateMusic("29.mp3", introMusicTimer / 8)
		end
	end

	if extro.started then
		playerFay.screenShakeTimer = 1
		playerCorn.screenShakeTimer = 1

		extro.timer = extro.timer - deltaTime
		if extro.timer < 0 then
			unregisterGameState()
			registerBossFightState()
		end
	end

	updateView(deltaTime)
	
	if (playerFay.isDead and playerFay.numDaisies <= 0) or (playerCorn.isDead and playerCorn.numDaisies <= 0) then
		gameOver.timer = gameOver.timer + deltaTime
		
		if not gameOver.musicStarted and gameOver.timer > 2 then
			gameOver.musicStarted = true
			daisy.playMusic("mus_defeat.mp3")
		end
	end

end

-- RRRRRRRR    EEEEEEEEEE  NN      NN
-- RR      RR  EE          NNNN    NN
-- RR      RR  EE          NNNN    NN
-- RRRRRRRR    EEEEEE      NN  NN  NN
-- RR      RR  EE          NN    NNNN
-- RR      RR  EE          NN    NNNN
-- RR      RR  EEEEEEEEEE  NN      NN D E R

local function renderNormal(view, sprite, x, y, scale, angle, alpha, red, green, blue)
	daisy.renderSpriteState(sprite, x - view.x, y - view.y, scale, angle, alpha, red, green, blue)
end
local function renderSpecial(view, sprite, x, y, scale, angle, alpha, red, green, blue)
	daisy.renderSpriteStateFay(sprite, y - view.x, x - view.y, scale, angle, alpha, red, green, blue)
end

local function renderCornPlayer(view, renderMethod)
	if playerCorn.isDead then
		renderMethod(view, cornSprite.rip, playerCorn.x - 24, playerCorn.y - 44, 1, 0, 255, 255, 255, 255)
		return
	end

	--CORN RENDER MUHAHAHA
	local alpha = 255
	if playerCorn.hurtTimer > 0 then
		alpha = 128
	end	
	local d = 0
	local a = 0
	local ymod = -16
	
	if playerCorn.animDirection == 0 then
		d = 15
		a = 0.65*math.pi
		local xOff = math.cos(a - playerCorn.leaning) * d
		local yOff = -math.sin(a - playerCorn.leaning) * d
		
		renderMethod(view, cornSprite.lWing.front, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, math.sin(playerCorn.animFlap / 15.0) * 0.2 * math.pi + playerCorn.leaning, alpha, 255, 255, 255)
		renderMethod(view, cornSprite.lWing.front, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, math.sin(playerCorn.animFlap / 15.0 + 0.2) * 0.2 * math.pi - 0.2 * math.pi + playerCorn.leaning, alpha, 255, 255, 255)

		d = 15
		a = 0.35*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.rWing.front, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, -1*math.sin(playerCorn.animFlap/15.0)*0.2*math.pi +playerCorn.leaning, alpha, 255, 255, 255)
		renderMethod(view, cornSprite.rWing.front, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, -1*math.sin(playerCorn.animFlap/15.0+0.2)*0.2*math.pi+0.2*math.pi+playerCorn.leaning, alpha, 255, 255, 255)

		d = 12
		a = 1.25*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.lLeg.front, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, 1*math.sin(playerCorn.animLimbs/95.0) + playerCorn.leaning, alpha, 255, 255, 255)

		d = 12
		a = 1.75*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.rLeg.front, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, -1*math.sin(playerCorn.animLimbs/95.0) + playerCorn.leaning, alpha, 255, 255, 255)

		d = 13
		a = 0.30*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.rArm.front, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, -0.5+1*math.sin(playerCorn.animLimbs/95.0)*0.6 + playerCorn.leaning, alpha, 255, 255, 255)

		d = 13
		a = 0.70*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.lArm.front, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, 0.5-1*math.sin(playerCorn.animLimbs/95.0)*0.6 + playerCorn.leaning, alpha, 255, 255, 255)


		renderMethod(view, cornSprite.body.front, playerCorn.x, playerCorn.y + ymod, 1, playerCorn.leaning, alpha, 255, 255, 255)

		d = 12
		a = 0.5*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.head.front, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, playerCorn.leaning, alpha, 255, 255, 255)
	elseif playerCorn.animDirection == 1 then
		d = 15
		a = 0.65*math.pi
		local xOff = math.cos(a - playerCorn.leaning) * d
		local yOff = -math.sin(a - playerCorn.leaning) * d
		
		renderMethod(view, cornSprite.lWing.front, playerCorn.x + xOff - 5, playerCorn.y + ymod + yOff, 1, math.sin(playerCorn.animFlap / 15.0) * 0.2 * math.pi + playerCorn.leaning, alpha, 255, 255, 255)
		renderMethod(view, cornSprite.lWing.front, playerCorn.x + xOff - 5, playerCorn.y + ymod + yOff, 1, math.sin(playerCorn.animFlap / 15.0 + 0.2) * 0.2 * math.pi - 0.2 * math.pi + playerCorn.leaning, alpha, 255, 255, 255)

		d = 15
		a = 0.35*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.rWing.front, playerCorn.x + xOff - 5, playerCorn.y + ymod + yOff, 1, -1*math.sin(playerCorn.animFlap/15.0)*0.2*math.pi +playerCorn.leaning, alpha, 255, 255, 255)
		renderMethod(view, cornSprite.rWing.front, playerCorn.x + xOff - 5, playerCorn.y + ymod + yOff, 1, -1*math.sin(playerCorn.animFlap/15.0+0.2)*0.2*math.pi+0.2*math.pi+playerCorn.leaning, alpha, 255, 255, 255)

		d = 12
		a = 1.75*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.rLeg.side, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, -1*math.sin(playerCorn.animLimbs/95.0) + playerCorn.leaning, alpha, 255, 255, 255)

		d = 13
		a = 0.30*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.rArm.side, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, -0.5+1*math.sin(playerCorn.animLimbs/95.0)*0.6 + playerCorn.leaning, alpha, 255, 255, 255)

		renderMethod(view, cornSprite.body.side, playerCorn.x, playerCorn.y + ymod, 1, playerCorn.leaning, alpha, 255, 255, 255)

		d = 12
		a = 0.5*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.head.side, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, playerCorn.leaning, alpha, 255, 255, 255)
		
		d = 12
		a = 1.25*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.lLeg.side, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, 1*math.sin(playerCorn.animLimbs/95.0) + playerCorn.leaning, alpha, 255, 255, 255)
		
		d = 13
		a = 0.70*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.lArm.side, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, 0.5-1*math.sin(playerCorn.animLimbs/95.0)*0.6 + playerCorn.leaning, alpha, 255, 255, 255)
	else
		d = 15
		a = 0.65*math.pi
		local xOff = math.cos(a - playerCorn.leaning) * d
		local yOff = -math.sin(a - playerCorn.leaning) * d
		
		renderMethod(view, cornSprite.lWing.front, playerCorn.x + xOff + 5, playerCorn.y + ymod + yOff, 1, math.sin(playerCorn.animFlap / 15.0) * 0.2 * math.pi + playerCorn.leaning, alpha, 255, 255, 255)
		renderMethod(view, cornSprite.lWing.front, playerCorn.x + xOff + 5, playerCorn.y + ymod + yOff, 1, math.sin(playerCorn.animFlap / 15.0 + 0.2) * 0.2 * math.pi - 0.2 * math.pi + playerCorn.leaning, alpha, 255, 255, 255)

		d = 15
		a = 0.35*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.rWing.front, playerCorn.x + xOff + 5, playerCorn.y + ymod + yOff, 1, -1*math.sin(playerCorn.animFlap/15.0)*0.2*math.pi +playerCorn.leaning, alpha, 255, 255, 255)
		renderMethod(view, cornSprite.rWing.front, playerCorn.x + xOff + 5, playerCorn.y + ymod + yOff, 1, -1*math.sin(playerCorn.animFlap/15.0+0.2)*0.2*math.pi+0.2*math.pi+playerCorn.leaning, alpha, 255, 255, 255)

		d = 12
		a = 1.25*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.lLeg.side2, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, 1*math.sin(playerCorn.animLimbs/95.0) + playerCorn.leaning, alpha, 255, 255, 255)

		d = 13
		a = 0.70*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.lArm.side2, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, 0.5-1*math.sin(playerCorn.animLimbs/95.0)*0.6 + playerCorn.leaning, alpha, 255, 255, 255)

		renderMethod(view, cornSprite.body.side2, playerCorn.x, playerCorn.y + ymod, 1, playerCorn.leaning, alpha, 255, 255, 255)

		d = 12
		a = 0.5*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.head.side2, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, playerCorn.leaning, alpha, 255, 255, 255)
		
		d = 12
		a = 1.75*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.rLeg.side2, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, -1*math.sin(playerCorn.animLimbs/95.0) + playerCorn.leaning, alpha, 255, 255, 255)
		
		d = 13
		a = 0.30*math.pi
		xOff = math.cos(a - playerCorn.leaning) * d
		yOff = -math.sin(a - playerCorn.leaning) * d
		renderMethod(view, cornSprite.rArm.side2, playerCorn.x + xOff, playerCorn.y + ymod + yOff, 1, -0.5+1*math.sin(playerCorn.animLimbs/95.0)*0.6 + playerCorn.leaning, alpha, 255, 255, 255)
	end
end

local function fayLimb(limb, d, a, r)
	playerFay.limbs[limb].d = d
	playerFay.limbs[limb].a = a
	playerFay.limbs[limb].r = r
end

local function fayLimbRotate(limb, r)

	playerFay.limbs[limb].r = r
end
local function setFayOrder(head, body, larm, rarm, lleg, rleg)
	playerFay.limbs["head"].order = head
	playerFay.limbs["body"].order = body
	playerFay.limbs["lArm"].order = larm
	playerFay.limbs["rArm"].order = rarm
	playerFay.limbs["lLeg"].order = lleg
	playerFay.limbs["rLeg"].order = rleg
end

local function renderFayPlayer(view, renderMethod)
	--FAY RENDER
	
	if playerFay.isDead then
		renderMethod(view, faySprite.rip, playerFay.x - 26, playerFay.y - 39, 1, 0, 255, 255, 255, 255)
		return
	end
	
	if playerFay.touchesGround then
	
		setFayOrder(2,3,1,4,6,5)
	
		if playerFay.vx > 5 then
			playerFay.animDirection = "side"
			fayLimb("rLeg",18+math.abs(math.sin(playerFay.x*0.07)*1),0.47 * math.pi, math.cos(playerFay.x*0.07)*0.6)
			fayLimb("lLeg",19+math.abs(math.sin(playerFay.x*0.07 + math.pi)*2),0.54 * math.pi, math.cos(playerFay.x*0.07 + math.pi)*0.6)
			fayLimb("rArm",36,0.50 * math.pi, -0.1+math.cos(playerFay.x*0.07 + math.pi))
			fayLimb("body",28,0.5 * math.pi, 0)
			fayLimb("head",40,0.5 * math.pi-math.sin(playerFay.x*0.07)*0.01, 0)
			fayLimb("lArm",37,0.52 * math.pi, 0.1+math.cos(playerFay.x*0.07)*0.6)

		elseif playerFay.vx < -5 then
			playerFay.animDirection = "side2"
			fayLimb("rLeg",18+math.abs(math.sin(playerFay.x*0.07)*1),0.47 * math.pi, math.cos(playerFay.x*0.07)*0.6)
			fayLimb("lLeg",19+math.abs(math.sin(playerFay.x*0.07 + math.pi)*2),0.54 * math.pi, math.cos(playerFay.x*0.07 + math.pi)*0.6)
			fayLimb("rArm",36,0.50 * math.pi, -0.1+math.cos(playerFay.x*0.07 + math.pi))
			fayLimb("body",28,0.5 * math.pi, 0)
			fayLimb("head",40,0.5 * math.pi-math.sin(playerFay.x*0.07)*0.01, 0)
			fayLimb("lArm",37,0.52 * math.pi, 0.1+math.cos(playerFay.x*0.07)*0.6)

		else
			playerFay.animDirection = "front"
			fayLimb("rLeg",18,0.43 * math.pi, 0)
			fayLimb("lLeg",17,0.55 * math.pi, 0)
			fayLimb("rArm",36,0.47 * math.pi, -0.1)
			fayLimb("body",28,0.5 * math.pi, 0)
			fayLimb("head",40,0.5 * math.pi, 0)
			fayLimb("lArm",37,0.54 * math.pi, 0.1)
		end

	else
		-- not on ground
		if playerFay.isAttached then
			playerFay.animDirection = "climb1"
			
			setFayOrder(1,2,3,4,5,6)
			
			local yScale = playerFay.y / 50.0
			yScale = math.cos((yScale - math.floor(yScale)) * math.pi * 2)
			if yScale < 0 then
				playerFay.animDirection = "climb2"
			end
			
			local xScale = playerFay.x / 50.0
			xScale = (xScale - math.floor(xScale))
			
			local armAngle = math.sin(xScale * math.pi * 2) * math.pi * .15
			
			fayLimb("rLeg",18,0.43 * math.pi, 0)
			fayLimb("lLeg",17,0.55 * math.pi, 0)
			fayLimb("rArm",36,0.47 * math.pi, armAngle)
			fayLimb("lArm",37,0.54 * math.pi, -armAngle)
			fayLimb("body",28,0.5 * math.pi, 0)
			fayLimb("head",40,0.5 * math.pi, 0)
			
		elseif playerFay.facingLeft then
			playerFay.animDirection = "front2"
			
			setFayOrder(2,3,4,1,5,6)
			fayLimb("rLeg",18,0.43 * math.pi, -0.4+math.sin(playerFay.vy*0.003)*0.3)
			fayLimb("lLeg",17,0.55 * math.pi, 0.4-math.sin(playerFay.vy*0.003)*0.3)
			fayLimb("rArm",36,0.47 * math.pi, -0.9+math.sin(playerFay.vy*0.003)*0.6)
			fayLimb("body",28,0.5 * math.pi, 0)
			fayLimb("head",40,0.5 * math.pi, 0)
			fayLimb("lArm",37,0.54 * math.pi, 0.6-math.sin(playerFay.vy*0.003)*0.6)
		else
			playerFay.animDirection = "front"
			
			setFayOrder(2,3,1,4,6,5)
			fayLimb("rLeg",18,0.43 * math.pi, -0.4+math.sin(playerFay.vy*0.003)*0.3)
			fayLimb("lLeg",17,0.55 * math.pi, 0.4-math.sin(playerFay.vy*0.003)*0.3)
			fayLimb("rArm",36,0.47 * math.pi, -0.9+math.sin(playerFay.vy*0.003)*0.6)
			fayLimb("body",28,0.5 * math.pi, 0)
			fayLimb("head",40,0.5 * math.pi, 0)
			fayLimb("lArm",37,0.54 * math.pi, 0.6-math.sin(playerFay.vy*0.003)*0.6)
		end		
	
	end
	
	if playerFay.carriedKey ~= null and not playerFay.isAttached then
		fayLimbRotate("rArm", 1.0*math.pi)
		fayLimbRotate("lArm", 1.0*math.pi)
	end

	table.sort(playerFay.limbNames, 
		function(a,b)
			return playerFay.limbs[a].order > playerFay.limbs[b].order
		end
	)

	local alpha = 255
	if playerFay.hurtTimer > 0 then
		alpha = 128
	end
	
	for index,limbName in ipairs(playerFay.limbNames) do
		renderMethod(view, faySprite[limbName][playerFay.animDirection], playerFay.x + math.cos(playerFay.limbs[limbName].a-playerFay.leaning)*playerFay.limbs[limbName].d, playerFay.y - math.sin(playerFay.limbs[limbName].a-playerFay.leaning)*playerFay.limbs[limbName].d - 2, 1, playerFay.leaning + playerFay.limbs[limbName].r, alpha, 255, 255, 255)
	end
end

local function renderDoors(view, renderMethod)
	for index,door in pairs(doors) do
		renderMethod(view, doorSprites.doorUpper, door.x - 16, door.y - 32 - door.anim * 26, 1, 0, 255, 255, 255, 255)
		renderMethod(view, doorSprites.doorLower, door.x - 16, door.y + door.anim * 26, 1, 0, 255, 255, 255, 255)
		
		if not door.open then
			if door.kind == 2 then
				renderMethod(view, doorSprites.doorRed, door.x, door.y, 1, 0, 255, 255, 255, 255)
			elseif door.kind == 3 then
				renderMethod(view, doorSprites.doorGreen, door.x, door.y, 1, 0, 255, 255, 255, 255)
			elseif door.kind == 4 then
				renderMethod(view, doorSprites.doorBlue, door.x, door.y, 1, 0, 255, 255, 255, 255)
			elseif door.kind == 5 then
				renderMethod(view, doorSprites.doorYellow, door.x, door.y, 1, 0, 255, 255, 255, 255)
			end
		end
		
		renderMethod(view, doorSprites.gear, door.x - 15, door.y - 32, 1, door.anim * -math.pi, 255, 255, 255, 255)
		renderMethod(view, doorSprites.gear, door.x + 15, door.y - 32, 1, door.anim * math.pi, 255, 255, 255, 255)
		renderMethod(view, doorSprites.gear, door.x - 15, door.y + 32, 1, door.anim * -math.pi, 255, 255, 255, 255)
		renderMethod(view, doorSprites.gear, door.x + 15, door.y + 32, 1, door.anim * math.pi, 255, 255, 255, 255)
		
		renderMethod(view, doorSprites.frameUpper, door.x, door.y - 32, 1, 0, 255, 255, 255, 255)
		renderMethod(view, doorSprites.frameLower, door.x, door.y + 32, 1, 0, 255, 255, 255, 255)
	end
end

local function renderDoodads(view, renderMethod)

	for index,doodad in pairs(doodads) do
	
		renderMethod(view, doodad.sprite, doodad.x, doodad.y, 1, 0, 255, 255, 255, 255)
	
	end

end

local function renderPowerups(view, renderMethod)
	for index,powerup in pairs(powerups) do
		local offX = math.cos((powerupTimer + powerup.animationOffset) * .6) * 10
		local offY = math.sin((powerupTimer + powerup.animationOffset) * .9) * 7
		local size = 1.0 + math.abs(math.cos((powerupTimer + powerup.animationOffset) * .23)) * .2
		local angle = math.sin((powerupTimer + powerup.animationOffset) * 1.2) * .2
		
		renderMethod(view, powerupSprites.glow, powerup.x + offX, powerup.y + offY, size, 0, 255, 255, 255, 255)
		renderMethod(view, powerupSprites[powerup.type], powerup.x + offX, powerup.y + offY, size, angle, 255, 255, 255, 255)
	end
end

local function renderKeys(view, renderMethod)
	for index,key in pairs(keys) do
		local kind = key.type
		if kind == 2 then
			renderMethod(view, doorSprites.keyRed, key.posX, key.posY, 1, key.angle, 255, 255, 140, 80)
		elseif kind == 3 then
			renderMethod(view, doorSprites.keyGreen, key.posX, key.posY, 1, key.angle, 255, 192, 255, 80)
		elseif kind == 4 then
			renderMethod(view, doorSprites.keyBlue, key.posX, key.posY, 1, key.angle, 255, 192, 140, 255)
		elseif kind == 5 then
			renderMethod(view, doorSprites.keyYellow, key.posX, key.posY, 1, key.angle, 255, 255, 255, 80)
		end
	end
end

local function renderJumpers(view, renderMethod)
	for index,jumper in pairs(jumpers) do
		renderMethod(view, jumperSprites.stalk, jumper.posX, jumper.posY, 1, 0, 255, 255, 255, 255)
		renderMethod(view, jumperSprites.head, jumper.posX, jumper.posY, 1, 0, 255, 255, 255, 255)
	end
end

local function renderCornShots(view, renderMethod)
	for index,shot in pairs(cornShots) do
		renderMethod(view, cornSprite.magicMissile, shot.x, shot.y, 1, shot.angle, 255, 255, 255, 255)
	end
end

local function renderMonsters(view, renderMethod)
	local monsters = getMonsters()
	for index,monster in pairs(monsters) do
		if monster.active then
			monster.kind.render(monster, view, renderMethod)
		end
	end
end

local function renderEffects(view, defaultRender, rotatedRender)
	for index,fx in pairs(fxItems) do
		if not fx.rotated then
			defaultRender(view, fx.sprite, fx.x, fx.y, fx.size, fx.spin, fx.alpha, 255, 255, 255)
		else
			rotatedRender(view, fx.sprite, fx.x, fx.y, fx.size, fx.spin, fx.alpha, 255, 255, 255)
		end
	end
end

local function renderPlayerView(player, field, fieldWidth, fieldHeight, left, top, width, height)
	player.viewX = player.x - width * .5
	player.viewY = player.y - height * .5

	if player.screenShakeTimer > 0 then
		player.viewX = player.viewX + math.random(-7, 7)
		player.viewY = player.viewY + math.random(-7, 7)
	end

	if player.viewX < -64 then
		player.viewX = -64
	elseif (player.viewX + (width - 64)) > (fieldWidth * 32) then
		player.viewX = (fieldWidth * 32) - (width - 64)
	end
	if player.viewY < -128 then
		player.viewY = -128
	elseif (player.viewY + height) > (fieldHeight * 32) then
		player.viewY = (fieldHeight * 32) - height
	end

	daisy.setScissorClip(left, top, left + width, top + height)

	do
		local bgX = (width - 1024) * (player.viewX + 64) / (((DIMA + 4) * 32) - width)
		local bgY = (height - 1024) * (player.viewY + 128) / (((DIMB + 4) * 32) - height)
		daisy.renderSpriteState(bgSprite, bgX, bgY, 2)
	end

	local sx,sy = math.floor(player.viewX / 32) + 1, math.floor(player.viewY / 32) + 1
	local ex,ey = sx + (width / 32) + 1, sy + (height / 32) + 1

	local ox,oy = player.viewX % 32 - left, player.viewY % 32 - top
	if player.viewX < 0 then
		ox = player.viewX - left
	end
	if player.viewY < 0 then
		oy = player.viewY - top
	end

	if sx < 1 then
		sx = 1
	end
	if sy < 1 then
		sy = 1
	end
	if ex > fieldWidth then
		ex = fieldWidth
	end
	if ey > fieldHeight then
		ey = fieldHeight
	end
	for y=sy,ey do
		for x=sx,ex do
			local list = gameFieldSprites[x][y]
			for i=1,#list do
				daisy.renderSpriteState(list[i], (x - sx) * 32 - ox, (y - sy) * 32 - oy)
			end
		end
	end
	
	local viewInfo = {}
	viewInfo.x = player.viewX - left
	viewInfo.y = player.viewY - top
	
	renderDoors(viewInfo, renderNormal)
	renderDoodads(viewInfo, renderNormal)

	renderJumpers(viewInfo, renderNormal)
	renderKeys(viewInfo, renderNormal)
	renderPowerups(viewInfo, renderNormal)
	renderMonsters(viewInfo, renderNormal)
	renderCornShots(viewInfo, renderSpecial)
	
	renderFayPlayer(viewInfo, renderNormal)
	renderCornPlayer(viewInfo, renderSpecial)

	renderEffects(viewInfo, renderNormal, renderSpecial)
	
	if player.help.timer > 0 then
		local scale = (7 - player.help.timer) / .75
		if scale > 1 then
			scale = 1
		end
		daisy.renderRectangle(player.x - scale * player.help.width * .5 - viewInfo.x, player.y + 10 - viewInfo.y, scale * player.help.width, 20, 190, 0, 0, 0)
		
		if scale >= .99 then
			daisy.renderText(player.help.text, player.x - viewInfo.x, player.y + 14 - viewInfo.y, 1, "gameFont.fnt")
		end
	end

	-- render health
	do
		local health = player.health
		local x = 60
		local anim = player.healthAnimation - math.floor(player.healthAnimation)
		local animIntensity = 255
		if anim < .5 then
			animIntensity = 255 - math.sin((anim / .5) * math.pi) * 160
		end
		while health > 1 do
			if health == 2 then
				daisy.renderSpriteState(faySprite.heart, left + x, top + 32, 1, 0, 255, animIntensity, animIntensity, animIntensity)
			else
				daisy.renderSpriteState(faySprite.heart, left + x, top + 32)
			end
			x = x + 44
			health = health - 2
		end
		if health > 0 then
			daisy.renderSpriteState(faySprite.heart, left + x, top + 32, .5, 0, 255, animIntensity, animIntensity, animIntensity)
		end
	end

	-- render daisies
	do
		local x = left + width - 40
		for i=1,player.numDaisies do
			if player.isDead and i == player.numDaisies then
				local scale = (player.hurtTimer / 5)
				local angle = scale * math.pi * 4.5
				daisy.renderSpriteState(powerupSprites[6], player.x - viewInfo.x + math.cos(angle) * 150 * scale, player.y - viewInfo.y + math.sin(angle) * 150 * scale, 1.5 - scale)
			else
				daisy.renderSpriteState(powerupSprites[6], x, top + 30)
				x = x - 30
			end
		end
	end

end

local function renderRotatedPlayerView(player, left, top, width, height)
	player.viewX = player.x - width * .5
	player.viewY = player.y - height * .5

	if player.screenShakeTimer > 0 then
		player.viewX = player.viewX + math.random(-7, 7)
		player.viewY = player.viewY + math.random(-7, 7)
	end

	if player.viewX < 0 then
		player.viewX = 0
	elseif (player.viewX + width) > (DIMB * 32) then
		player.viewX = (DIMB * 32) - width
	end
	if player.viewY < 0 then
		player.viewY = 0
	elseif (player.viewY + height) > (DIMA * 32) then
		player.viewY = (DIMA * 32) - height
	end

	daisy.setScissorClip(left, top, left + width, top + height)
	
	do
		local bgX = (width - 1024) * player.viewX / (((DIMA + 4) * 32) - width)
		local bgY = 0
		daisy.renderSpriteStateFay(bgSprite, bgX, bgY, 2)
	end

	local sx,sy = math.floor(player.viewX / 32) + 1, math.floor(player.viewY / 32) + 1
	local ex,ey = sx + (width / 32) + 1, sy + (height / 32) + 1

	local ox,oy = player.viewX % 32 - left, player.viewY % 32 - top
	if sx < 1 then
		sx = 1
	end
	if sy < 1 then
		sy = 1
	end
	if ex > DIMB then
		ex = DIMB
	end
	if ey > DIMA then
		ey = DIMA
	end
	for y=sy,ey do
		for x=sx,ex do
			local list = gameFieldSprites[y][x]
			local left,top = (x - sx) * 32 - ox, (y - sy) * 32 - oy
			for i=1,#list do
				daisy.renderSpriteStateFreeShape(list[i], left, top, left, top + 32, left + 32, top, left + 32, top + 32)
			end
		end
	end

	local viewInfo = {}
	viewInfo.x = player.viewX - left
	viewInfo.y = player.viewY - top
	
	renderDoors(viewInfo, renderSpecial)
	renderDoodads(viewInfo, renderSpecial)
	
	renderJumpers(viewInfo, renderSpecial)
	renderKeys(viewInfo, renderSpecial)
	renderPowerups(viewInfo, renderSpecial)
	renderMonsters(viewInfo, renderSpecial)
	renderCornShots(viewInfo, renderNormal)

	renderFayPlayer(viewInfo, renderSpecial)	
	renderCornPlayer(viewInfo, renderNormal)

	renderEffects(viewInfo, renderSpecial, renderNormal)

	if player.help.timer > 0 then
		local scale = (7 - player.help.timer) / .75
		if scale > 1 then
			scale = 1
		end
		daisy.renderRectangle(player.x - scale * player.help.width * .5 - viewInfo.x, player.y + 10 - viewInfo.y, scale * player.help.width, 20, 190, 0, 0, 0)
		
		if scale >= .99 then
			daisy.renderText(player.help.text, player.x - viewInfo.x, player.y + 14 - viewInfo.y, 1, "gameFont.fnt")
		end
	end

	-- render attack power
	do
		if player.powerRefreshTimer > 0 then
			local a = math.sin(player.powerRefreshTimer * math.pi * .5) * 255
			local s = 1.0 + math.sin(player.powerRefreshTimer * 16 * math.pi) * 0.03
			daisy.renderSpriteState(cornSprite.powerRefresh, 110, 80, s, 0, a)
		end
		local value = 0
		local x = 45
		local sprite = cornSprite.powerLeft
		while value <= player.powerMeter do
			if player.powerMeter >= (value + .5) then
				daisy.renderSpriteState(sprite, left + x, top + 80, 1)
			else
				local alpha = ((player.powerMeter - value) / .5) * 255
				daisy.renderSpriteState(sprite, left + x, top + 80, 1, 0, alpha)
			end
			value = value + .5
			x = x + 15
			if sprite == cornSprite.powerRight then
				x = x + 1
				sprite = cornSprite.powerLeft
			else
				sprite = cornSprite.powerRight
			end
		end
	end

	-- render health
	do
		local health = player.health
		local x = 60
		local anim = player.healthAnimation - math.floor(player.healthAnimation)
		local animScale = 1
		if anim < .3 then
			animScale = 1.0 - math.sin((anim / .3) * math.pi) * .25
		elseif anim < .5 then
			animScale = 1.0 - math.sin(((anim - .3) / .2) * math.pi) * .15
		end
		while health > 1 do
			if health == 2 then
				daisy.renderSpriteState(cornSprite.heart, left + x, top + 32, animScale)
			else
				daisy.renderSpriteState(cornSprite.heart, left + x, top + 32)
			end
			x = x + 40
			health = health - 2
		end
		if health > 0 then
			daisy.renderSpriteState(cornSprite.heart, left + x, top + 32, .65 * animScale)
		end
	end
	
	-- render daisies
	do
		local x = left + width - 40
		for i=1,player.numDaisies do
			if player.isDead and i == player.numDaisies then
				local scale = (player.hurtTimer / 5)
				local angle = scale * math.pi * 4.5
				daisy.renderSpriteState(powerupSprites[6], player.x - viewInfo.x + math.cos(angle) * 150 * scale, player.y - viewInfo.y + math.sin(angle) * 150 * scale, 1.5 - scale)
			else
				daisy.renderSpriteState(powerupSprites[6], x, top + 30)
				x = x - 30
			end
		end
	end
end


local function gameRender()
	local w,h = daisy.getScreenSize()
	local half = w * .5

	if singlePlayer then
		if screenInfo.switchPos > 0 and screenInfo.switchPos < 1 then

			local fayWidth = math.ceil(1024 * (1.0 - screenInfo.switchPos))

			renderPlayerView(playerFay, gameField, DIMA, DIMB, 0, 0, fayWidth, 768)
			renderRotatedPlayerView(playerCorn, fayWidth, 0, 1024 - fayWidth, 768)

			daisy.clearScissorClip()
			daisy.renderLine(fayWidth - 1, 0, fayWidth - 1, h, 255, 0, 0, 0)
			daisy.renderLine(fayWidth, 0, fayWidth, h, 255, 0, 0, 0)

		else
			if activePlayer == 0 then
				renderPlayerView(playerFay, gameField, DIMA, DIMB, 0, 0, 1024, 768)
			else
				renderRotatedPlayerView(playerCorn, 0, 0, 1024, 768)
			end
		end
	else
		renderPlayerView(playerFay, gameField, DIMA, DIMB, 0, 0, half, h)
		renderRotatedPlayerView(playerCorn, half, 0, half, h)

		daisy.clearScissorClip()
		daisy.renderLine(half - 1, 0, half - 1, h, 255, 0, 0, 0)
		daisy.renderLine(half, 0, half, h, 255, 0, 0, 0)
	end

	if extro.started then
		local alpha = (1.0 - extro.timer / 8.0) * 255
		if alpha > 0 then
			daisy.renderRectangle(0, 0, w, h, alpha, 48, 20, 76)
		end
	end
	
	if gameOver.timer > 2 then
		local scale = (gameOver.timer - 2)
		if scale > 1 then
			scale = 1
		end
		
		daisy.renderRectangle(0, 386 - scale * 70, 1024, scale * 140, 255, 0, 0, 0)
		daisy.renderSpriteState(circleSprite, 512, 386, scale * 10, 0, 255, 0, 0, 0)
		
		if gameOver.timer > 3 then
			daisy.renderText("Game Over", 512, 320, 1, "gameFont.fnt")
			
			if playerFay.isDead then
				daisy.renderText("Fay has been defeated! Cornipolous is now doomed", 400, 360, 0)
				daisy.renderText("to live his eternal life in loneliness :(", 400, 375, 0)
			else
				daisy.renderText("Cornipolous has been defeated! Fay has no chance", 400, 360, 0)
				daisy.renderText("of escaping on her own, and is locked up in the", 400, 375, 0)
				daisy.renderText("tower for the rest of her life... :(", 400, 390, 0)
			end
			
			daisy.renderText("Press Shift+R to restart", 512, 430, 1)
		end
	end

	--daisy.renderText("Seed: " .. randomSeed, 512, 140, 2)
end

function registerGameState()
	randomSeed = os.time()
	math.randomseed(randomSeed)
	gameField,gameFieldRotated,DIMA,DIMB,gameFieldSprites = createWorld(keys, jumpers, doors, doodads)
	makePlayers()
	recountPlayerDaisies()
	initMonsters(gameField, playerFay, playerCorn, cornShots, powerups)
	
	playerCorn.help.text = "Use Down Key to Attack"
	playerCorn.help.timer = 7
	playerCorn.help.width = 220

	--blockSprite = blockSprite or daisy.createSpriteState("Square", "itt.dat")
	jumperSprites.stalk = blockSprite or daisy.createSpriteState("JumpStalk", "tiles.dat")
	jumperSprites.head = blockSprite or daisy.createSpriteState("JumpHead", "tiles.dat")
	circleSprite = circleSprite or daisy.createSpriteState("Circle", "itt.dat")
	bgSprite = bgSprite or daisy.createSpriteState("Background", "gfx.dat")

	fxSprite.blood = daisy.createSpriteState("blood", "monsters.dat")
	
	faySprite.body = {front = daisy.createSpriteState("fayBodyFront", "players.dat"), front2 = daisy.createSpriteState("fayBodyFront2", "players.dat"), 
		side = daisy.createSpriteState("fayBodySide", "players.dat"), side2 = daisy.createSpriteState("fayBodySide2", "players.dat"),
		climb1 = daisy.createSpriteState("fayBodyClimb1", "players.dat"), climb2 = daisy.createSpriteState("fayBodyClimb2", "players.dat")}
	faySprite.head = {front = daisy.createSpriteState("fayHeadFront", "players.dat"), front2 = daisy.createSpriteState("fayHeadFront2", "players.dat"), 
		side = daisy.createSpriteState("fayHeadSide", "players.dat"), side2 = daisy.createSpriteState("fayHeadSide2", "players.dat"),
		climb1 = daisy.createSpriteState("fayHeadClimb1", "players.dat"), climb2 = daisy.createSpriteState("fayHeadClimb2", "players.dat")}
	faySprite.lArm = {front = daisy.createSpriteState("fayLArmFront", "players.dat"), front2 = daisy.createSpriteState("fayLArmFront2", "players.dat"), 
		side = daisy.createSpriteState("fayLArmSide", "players.dat"), side2 = daisy.createSpriteState("fayLArmSide2", "players.dat"),
		climb1 = daisy.createSpriteState("fayLArmClimb1", "players.dat"), climb2 = daisy.createSpriteState("fayLArmClimb2", "players.dat")}
	faySprite.rArm = {front = daisy.createSpriteState("fayRArmFront", "players.dat"), front2 = daisy.createSpriteState("fayRArmFront2", "players.dat"), 
		side = daisy.createSpriteState("fayRArmSide", "players.dat"), side2 = daisy.createSpriteState("fayRArmSide2", "players.dat"),
		climb1 = daisy.createSpriteState("fayRArmClimb1", "players.dat"), climb2 = daisy.createSpriteState("fayRArmClimb2", "players.dat")}
	faySprite.lLeg = {front = daisy.createSpriteState("fayLLegFront", "players.dat"), front2 = daisy.createSpriteState("fayLLegFront2", "players.dat"), 
		side = daisy.createSpriteState("fayLLegSide", "players.dat"), side2 = daisy.createSpriteState("fayLLegSide2", "players.dat"),
		climb1 = daisy.createSpriteState("fayLLegClimb1", "players.dat"), climb2 = daisy.createSpriteState("fayLLegClimb2", "players.dat")} 
	faySprite.rLeg = {front = daisy.createSpriteState("fayRLegFront", "players.dat"), front2 = daisy.createSpriteState("fayRLegFront2", "players.dat"), 
		side = daisy.createSpriteState("fayRLegSide", "players.dat"), side2 = daisy.createSpriteState("fayRLegSide2", "players.dat"),
		climb1 = daisy.createSpriteState("fayRLegClimb1", "players.dat"), climb2 = daisy.createSpriteState("fayRLegClimb2", "players.dat")}

	cornSprite["body"] = {front = daisy.createSpriteState("cornBodyFront", "players.dat"),side = daisy.createSpriteState("cornBodySide", "players.dat"), side2 = daisy.createSpriteState("cornBodySide2", "players.dat")}
	cornSprite["head"] = {front = daisy.createSpriteState("cornHeadFront", "players.dat") ,side =daisy.createSpriteState("cornHeadSide", "players.dat"), side2 = daisy.createSpriteState("cornHeadSide2", "players.dat")}
	cornSprite["lArm"] = {front = daisy.createSpriteState("cornLArmFront", "players.dat"),side =daisy.createSpriteState("cornLArmSide", "players.dat"), side2 = daisy.createSpriteState("cornLArmSide2", "players.dat")}
	cornSprite["rArm"] = {front = daisy.createSpriteState("cornRArmFront", "players.dat"),side = daisy.createSpriteState("cornRArmSide", "players.dat"), side2 = daisy.createSpriteState("cornRArmSide2", "players.dat")}
	cornSprite["rLeg"] = {front = daisy.createSpriteState("cornRLegFront", "players.dat"), side = daisy.createSpriteState("cornRLegSide", "players.dat"), side2 = daisy.createSpriteState("cornRLegSide2", "players.dat")}
	cornSprite["lLeg"] = {front = daisy.createSpriteState("cornLLegFront", "players.dat"), side = daisy.createSpriteState("cornLLegSide", "players.dat"), side2 = daisy.createSpriteState("cornLLegSide2", "players.dat")}
	cornSprite["lWing"] = {front = daisy.createSpriteState("cornLWingFront", "players.dat"), side = daisy.createSpriteState("cornLWingSide", "players.dat")}
	cornSprite["rWing"] = {front = daisy.createSpriteState("cornRWingFront", "players.dat"), side = daisy.createSpriteState("cornRWingSide", "players.dat")}
	cornSprite.magicHit = daisy.createSpriteState("cornMagicHit", "players.dat")
	cornSprite.magicGlow = daisy.createSpriteState("cornMagicGlow", "players.dat")
	cornSprite.magicMissile = daisy.createSpriteState("cornMagicMissile", "players.dat")
	cornSprite.heart = daisy.createSpriteState("cornGuiHeart", "players.dat")
	cornSprite.powerLeft = daisy.createSpriteState("cornGuiAmmoL", "players.dat")
	cornSprite.powerRight = daisy.createSpriteState("cornGuiAmmoR", "players.dat")
	cornSprite.powerRefresh = daisy.createSpriteState("cornGuiBloodLust", "players.dat")
	
	cornSprite.rip = daisy.createSpriteState("CornTomb", "tiles.dat")
	faySprite.rip = daisy.createSpriteState("FayTomb", "tiles.dat")

	faySprite.heart = daisy.createSpriteState("fayGuiHeart", "players.dat")
	
	doorSprites.frameUpper = daisy.createSpriteState("doorCoverTop", "stuff.dat")
	doorSprites.frameLower = daisy.createSpriteState("doorCoverBottom", "stuff.dat")
	doorSprites.doorUpper = daisy.createSpriteState("doorTop", "stuff.dat")
	doorSprites.doorLower = daisy.createSpriteState("doorBottom", "stuff.dat")
	doorSprites.gear = daisy.createSpriteState("doorGear", "stuff.dat")
	doorSprites.doorYellow = daisy.createSpriteState("doorGlowYellow", "stuff.dat")
	doorSprites.doorGreen = daisy.createSpriteState("doorGlowGreen", "stuff.dat")
	doorSprites.doorBlue = daisy.createSpriteState("doorGlowBlue", "stuff.dat")
	doorSprites.doorRed = daisy.createSpriteState("doorGlowRed", "stuff.dat")
	doorSprites.keyYellow = daisy.createSpriteState("keyYellow", "stuff.dat")
	doorSprites.keyGreen = daisy.createSpriteState("keyGreen", "stuff.dat")
	doorSprites.keyBlue = daisy.createSpriteState("keyBlue", "stuff.dat")
	doorSprites.keyRed = daisy.createSpriteState("KeyRed", "stuff.dat")

	hook.add("frameUpdate", gameRun)
	hook.add("frameRender", gameRender)
end

function unregisterGameState()
	hook.remove("frameUpdate", gameRun)
	hook.remove("frameRender", gameRender)
end
